from __future__ import unicode_literals
from collections import namedtuple
import re
try:
    from urllib.parse import urljoin, urlencode
except ImportError:
    from urllib import urlencode
    from urlparse import urljoin

from django.conf import settings
from django.contrib.sites.models import Site
from django.core.exceptions import ImproperlyConfigured
from django.db.models import get_model

PAYMENT_VARIANTS = {
    'default': ('payments.dummy.DummyProvider', {})}

PAYMENT_HOST = getattr(settings, 'PAYMENT_HOST', None)
PAYMENT_USES_SSL = getattr(settings, 'PAYMENT_USES_SSL', False)

if not PAYMENT_HOST:
    if not 'django.contrib.sites' in settings.INSTALLED_APPS:
        raise ImproperlyConfigured('The PAYMENT_HOST setting without '
                                   'the sites app must not be empty.')

PurchasedItem = namedtuple('PurchasedItem',
                           'name, quantity, price, currency, sku')


def get_base_url():
    protocol = 'https' if PAYMENT_USES_SSL else 'http'
    if not PAYMENT_HOST:
        current_site = Site.objects.get_current()
        domain = current_site.domain
        return '%s://%s' % (protocol, domain)
    return '%s://%s' % (protocol, PAYMENT_HOST)


class RedirectNeeded(Exception):
    pass


class PaymentError(Exception):
    pass


class ExternalPostNeeded(Exception):
    pass


class BasicProvider(object):
    '''
    This class defines the provider API. It should not be instantiated
    directly. Use factory instead.
    '''
    _method = 'post'

    def _action(self):
        return self.get_return_url()
    _action = property(_action)

    def __init__(self, payment, capture=True):
        self._capture = capture
        self.payment = payment

    def get_hidden_fields(self):
        '''
        Converts a payment into a dict containing transaction data. Use
        get_form instead to get a form suitable for templates.

        When implementing a new payment provider, overload this method to
        transfer provider-specific data.
        '''
        raise NotImplementedError()

    def get_form(self, data=None):
        '''
        Converts *payment* into a form suitable for Django templates.
        '''
        from .forms import PaymentForm
        return PaymentForm(self.get_hidden_fields(),
                           self._action, self._method)

    def process_data(self, request):
        '''
        Process callback request from a payment provider.
        '''
        raise NotImplementedError()

    def get_token_from_request(self, request):
        '''
        Return payment token from provider request.
        '''
        raise NotImplementedError()

    def get_return_url(self, extra_data=None):
        payment_link = self.payment.get_process_url()
        url = urljoin(get_base_url(), payment_link)
        if extra_data:
            qs = urlencode(extra_data)
            return url + '?' + qs
        return url

    def capture(self, amount=None):
        raise NotImplementedError()

    def release(self):
        raise NotImplementedError()

    def refund(self):
        raise NotImplementedError()


def provider_factory(variant, payment=None):
    '''
    Return the provider instance based on variant
    '''
    variants = getattr(settings, 'PAYMENT_VARIANTS', PAYMENT_VARIANTS)
    handler, config = variants.get(variant, (None, None))
    if not handler:
        raise ValueError('Payment variant does not exist: %s' %
                         (variant,))
    path = handler.split('.')
    if len(path) < 2:
        raise ValueError('Payment variant uses an invalid payment module: %s' %
                         (variant,))
    module_path = str('.'.join(path[:-1]))
    class_name = str(path[-1])
    module = __import__(module_path, globals(), locals(), [class_name])
    class_ = getattr(module, class_name)
    return class_(payment, **config)


def factory(payment):
    '''
    Takes the payment object and returns an appropriate provider instance.
    '''
    return provider_factory(payment.variant, payment)


def get_payment_model():
    '''
    Return the Payment model that is active in this project
    '''
    try:
        app_label, model_name = settings.PAYMENT_MODEL.split('.')
    except (ValueError, AttributeError):
        raise ImproperlyConfigured('PAYMENT_MODEL must be of the form '
                                   '"app_label.model_name"')
    payment_model = get_model(app_label, model_name)
    if payment_model is None:
        msg = (
            'PAYMENT_MODEL refers to model "%s" that has not been installed' %
            settings.PAYMENT_MODEL)
        raise ImproperlyConfigured(msg)
    return payment_model


CARD_TYPES = [
    (r'^4[0-9]{12}(?:[0-9]{3})?$', 'visa', 'VISA'),
    (r'^5[1-5][0-9]{14}$', 'mastercard', 'MasterCard'),
    (r'^6(?:011|5[0-9]{2})[0-9]{12}$', 'discover', 'Discover'),
    (r'^3[47][0-9]{13}$', 'amex', 'American Express'),
    (r'^(?:(?:2131|1800|35\d{3})\d{11})$', 'jcb', 'JCB'),
    (r'^(?:3(?:0[0-5]|[68][0-9])[0-9]{11})$', 'diners', 'Diners Club'),
    (r'^(?:5[0678]\d\d|6304|6390|67\d\d)\d{8,15}$', 'maestro', 'Maestro')]


def get_credit_card_issuer(number):
    for regexp, card_type, name in CARD_TYPES:
        if re.match(regexp, number):
            return card_type, name
    return None, None



from __future__ import unicode_literals
try:
    # For Python 3.0 and later
    from urllib.error import URLError
    from urllib.parse import urlencode
except ImportError:
    # Fall back to Python 2's urllib2
    from urllib2 import URLError
    from urllib import urlencode
from django.http import HttpResponseRedirect

from .forms import DummyForm
from .. import BasicProvider, RedirectNeeded, PaymentError


class DummyProvider(BasicProvider):
    '''
    Dummy payment provider
    '''

    def get_form(self, data=None):
        if self.payment.status == 'waiting':
            self.payment.change_status('input')
        form = DummyForm(data=data, hidden_inputs=False, provider=self,
                         payment=self.payment)
        if form.is_valid():
            new_status = form.cleaned_data['status']
            self.payment.change_status(new_status)
            new_fraud_status = form.cleaned_data['fraud_status']
            self.payment.change_fraud_status(new_fraud_status)

            gateway_response = form.cleaned_data.get('gateway_response')
            verification_result = form.cleaned_data.get('verification_result')
            if gateway_response or verification_result:
                if gateway_response == '3ds-disabled':
                    # Standard request without 3DSecure
                    pass
                elif gateway_response == '3ds-redirect':
                    # Simulate redirect to 3DS and get back to normal
                    # payment processing
                    process_url = self.payment.get_process_url()
                    params = urlencode(
                        {'verification_result': verification_result})
                    redirect_url = '%s?%s' % (process_url, params)
                    raise RedirectNeeded(redirect_url)
                elif gateway_response == 'failure':
                    # Gateway raises error (HTTP 500 for example)
                    raise URLError('Opps')
                elif gateway_response == 'payment-error':
                    raise PaymentError('Unsupported operation')

            if new_status in ['preauth', 'confirmed']:
                raise RedirectNeeded(self.payment.get_success_url())
            raise RedirectNeeded(self.payment.get_failure_url())
        return form

    def process_data(self, request):
        verification_result = request.GET.get('verification_result')
        if verification_result:
            self.payment.change_status(verification_result)
        if self.payment.status in ['confirmed', 'preauth']:
            return HttpResponseRedirect(self.payment.get_success_url())
        return HttpResponseRedirect(self.payment.get_failure_url())

    def capture(self, amount=None):
        self.payment.change_status('confirmed')
        return amount

    def release(self):
        return None

    def refund(self, amount=None):
        return amount or 0



from __future__ import unicode_literals
from decimal import Decimal
from unittest import TestCase

from . import StripeProvider


class Payment(object):

    id = 1
    description = 'payment'
    currency = 'USD'
    delivery = Decimal(10)
    status = 'waiting'
    tax = Decimal(10)
    total = Decimal(100)

    def change_status(self, status):
        self.status = status

    def get_failure_url(self):
        return 'http://cancel.com'

    def get_process_url(self):
        return 'http://example.com'

    def get_purchased_items(self):
        return []

    def save(self):
        return self

    def get_success_url(self):
        return 'http://success.com'


class TestStripeProvider(TestCase):

    def test_form(self):
        payment = Payment()
        provider = StripeProvider(
            payment=payment, name='Example.com store',
            secret_key='123', public_key='abc')
        form = provider.get_form()
        self.assertTrue(
            '<script class="stripe-button" data-amount="10000" data-currency="USD" data-description="payment" data-image="" data-key="abc" data-name="Example.com store" src="https://checkout.stripe.com/checkout.js"></script>' in str(form))



from __future__ import unicode_literals
from decimal import Decimal
import json
from unittest import TestCase

from . import CyberSourceProvider
from .. import PurchasedItem


CLIENT_ID = 'abc123'
PAYMENT_TOKEN = '5a4dae68-2715-4b1e-8bb2-2c2dbe9255f6'
SECRET = '123abc'
VARIANT = 'wallet'


class Payment(object):

    id = 1
    description = 'payment'
    currency = 'AED'
    delivery = Decimal(10)
    status = 'waiting'
    tax = Decimal(10)
    token = PAYMENT_TOKEN
    total = Decimal(210)
    variant = VARIANT
    transaction_id = None

    billing_first_name = 'John'
    billing_last_name = 'Doe'
    billing_address_1 = 'Somewhere'
    billing_address_2 = 'Over the Rainbow'
    billing_city = 'Washington'
    billing_country_code = 'US'
    billing_country_area = 'District of Columbia'
    billing_postcode = '20505'
    billing_email = 'test@room-303.com'

    customer_ip_address = '82.196.81.11'

    def change_status(self, status):
        self.status = status

    def get_failure_url(self):
        return 'http://cancel.com'

    def get_process_url(self):
        return 'http://example.com'

    def get_purchased_items(self):
        return [
            PurchasedItem(
                name='foo', quantity=Decimal('10'), price=Decimal('20'),
                currency='AED', sku='bar')]

    def get_success_url(self):
        return 'http://success.com'


class TestCyberSourceProvider(TestCase):

    def test_purchase(self):
        payment = Payment()
        provider = CyberSourceProvider(
            payment,
            capture=False,
            merchant_id='NI_MAGRUDY_AED',
            password='2DROk7NEWI2nrjKzG5LGjm1kMeN955MhVXFezc/1wzMi6tTmQyuSyxVWZveMlqi3PAuqIST+q9JQ4882JDH8b6xVlkMUmXGXdu7I4kfBeNNitAHmi7Kd4U5z9NPmzXExt+suXMTVI8rctpLP/cAv3qGQZVZSeVLBm/LET8I7W9/yi6NwPJV4pslSCVoFy5xu/Cjppeeh6d5rrK7HFioV3GKt15opCNwd9J/RC+Iw/ixQJ2qnVD4ELKQbwTaamzEYxxYH2ob4RIudESeGNURIIMRtopDeaLk8PgwpGG7sHC33g58e1UMe9cDQv55iO+ONWCHCsgLXzuerN5E6WEVBLQ==')
        # provider.refund()
        form = provider.get_form({
            'expiration_0': '8',
            'expiration_1': '2014',
            'name': 'Tester',
            'number': '4111111111111111',
            'cvv2': '123'})
        form.clean()
        print form.errors
        print form.cleaned_data



from __future__ import unicode_literals
import json
from uuid import uuid4

from django.conf import settings
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.translation import ugettext_lazy as _

from . import factory

DEFAULT_PAYMENT_STATUS_CHOICES = (
    ('waiting', _('Waiting for confirmation')),
    ('preauth', _('Pre-authorized')),
    ('confirmed', _('Confirmed')),
    ('rejected', _('Rejected')),
    ('refunded', _('Refunded')),
    ('error', _('Error')),
    ('input', _('Input'))
)
PAYMENT_STATUS_CHOICES = getattr(settings, 'PAYMENT_STATUS_CHOICES',
                                 DEFAULT_PAYMENT_STATUS_CHOICES)

FRAUD_CHOICES = (
    ('unknown', _('Unknown')),
    ('accept', _('Passed')),
    ('reject', _('Rejected')),
    ('review', _('Review')))


class PaymentAttributeProxy(object):

    def __init__(self, payment):
        self._payment = payment
        super(PaymentAttributeProxy, self).__init__()

    def __getattr__(self, item):
        data = json.loads(self._payment.extra_data or '{}')
        return data[item]

    def __setattr__(self, key, value):
        if key == '_payment':
            return super(PaymentAttributeProxy, self).__setattr__(key, value)
        try:
            data = json.loads(self._payment.extra_data)
        except ValueError:
            data = {}
        data[key] = value
        self._payment.extra_data = json.dumps(data)


class BasePayment(models.Model):
    '''
    Represents a single transaction. Each instance has one or more PaymentItem.
    '''
    variant = models.CharField(max_length=255)
    #: Transaction status
    status = models.CharField(
        max_length=10, choices=PAYMENT_STATUS_CHOICES, default='waiting')
    fraud_status = models.CharField(
        _('fraud check'), max_length=10, choices=FRAUD_CHOICES,
        default='unknown')
    fraud_message = models.TextField(blank=True, default='')
    #: Creation date and time
    created = models.DateTimeField(auto_now_add=True)
    #: Date and time of last modification
    modified = models.DateTimeField(auto_now=True)
    #: Transaction ID (if applicable)
    transaction_id = models.CharField(max_length=255, blank=True)
    #: Currency code (may be provider-specific)
    currency = models.CharField(max_length=10)
    #: Total amount (gross)
    total = models.DecimalField(max_digits=9, decimal_places=2, default='0.0')
    delivery = models.DecimalField(
        max_digits=9, decimal_places=2, default='0.0')
    tax = models.DecimalField(max_digits=9, decimal_places=2, default='0.0')
    description = models.TextField(blank=True, default='')
    billing_first_name = models.CharField(max_length=256, blank=True)
    billing_last_name = models.CharField(max_length=256, blank=True)
    billing_address_1 = models.CharField(max_length=256, blank=True)
    billing_address_2 = models.CharField(max_length=256, blank=True)
    billing_city = models.CharField(max_length=256, blank=True)
    billing_postcode = models.CharField(max_length=256, blank=True)
    billing_country_code = models.CharField(max_length=2, blank=True)
    billing_country_area = models.CharField(max_length=256, blank=True)
    billing_email = models.EmailField(blank=True)
    customer_ip_address = models.IPAddressField(blank=True)
    extra_data = models.TextField(blank=True, default='')
    message = models.TextField(blank=True, default='')
    token = models.CharField(max_length=36, blank=True, default='')
    captured_amount = models.DecimalField(
        max_digits=9, decimal_places=2, default='0.0')

    class Meta:
        abstract = True

    def change_status(self, status, message=''):
        '''
        Updates the Payment status and sends the status_changed signal.
        '''
        from .signals import status_changed
        self.status = status
        self.message = message
        self.save()
        status_changed.send(sender=type(self), instance=self)

    def change_fraud_status(self, status, message='', commit=True):
        available_statuses = [choice[0] for choice in FRAUD_CHOICES]
        if status not in available_statuses:
            raise ValueError(
                'Status should be one of: %s' % ', '.join(available_statuses))
        self.fraud_status = status
        self.fraud_message = message
        if commit:
            self.save()

    def save(self, *args, **kwargs):
        if not self.token:
            for _i in range(100):
                token = str(uuid4())
                if not type(self).objects.filter(token=token).exists():
                    self.token = token
                    break
        return super(BasePayment, self).save(*args, **kwargs)

    def __unicode__(self):
        return self.variant

    def get_form(self, data=None):
        provider = factory(self)
        return provider.get_form(data=data)

    def get_purchased_items(self):
        return []

    def get_failure_url(self):
        raise NotImplementedError()

    def get_success_url(self):
        raise NotImplementedError()

    def get_process_url(self):
        return reverse('process_payment', kwargs={'token': self.token})

    def capture(self, amount=None):
        if self.status != 'preauth':
            raise ValueError(
                'Only pre-authorized payments can be captured.')
        provider = factory(self)
        amount = provider.capture(amount)
        if amount:
            self.captured_amount = amount
            self.change_status('confirmed')

    def release(self):
        if self.status != 'preauth':
            raise ValueError(
                'Only pre-authorized payments can be released.')
        provider = factory(self)
        provider.release()
        self.change_status('refunded')

    def refund(self, amount=None):
        if self.status != 'confirmed':
            raise ValueError(
                'Only charged payments can be refunded.')
        if amount > self.captured_amount:
            raise ValueError(
                'Refund amount can not be greater then captured amount')
        provider = factory(self)
        amount = provider.refund(amount)
        self.captured_amount -= amount
        if self.captured_amount == 0 and self.status != 'refunded':
            self.change_status('refunded')
        self.save()

    @property
    def attrs(self):
        return PaymentAttributeProxy(self)



from __future__ import unicode_literals
from calendar import monthrange
from datetime import date
import re

from django import forms
from django.core import validators
from django.utils.translation import ugettext_lazy as _

from . import get_credit_card_issuer
from .widgets import CreditCardExpiryWidget, CreditCardNumberWidget


class CreditCardNumberField(forms.CharField):

    widget = CreditCardNumberWidget(attrs={'autocomplete': 'cc-number'})
    default_error_messages = {
        'invalid': _('Please enter a valid card number'),
        'invalid_type': _('We accept only %(valid_types)s')}

    def __init__(self, valid_types=None, *args, **kwargs):
        self.valid_types = valid_types
        kwargs['max_length'] = kwargs.pop('max_length', 32)
        super(CreditCardNumberField, self).__init__(*args, **kwargs)

    def to_python(self, value):
        if value is not None:
            value = re.sub('[\s-]+', '', value)
        return super(CreditCardNumberField, self).to_python(value)

    def validate(self, value):
        card_type, issuer_name = get_credit_card_issuer(value)
        if value in validators.EMPTY_VALUES and self.required:
            raise forms.ValidationError(self.error_messages['required'])
        if value and not self.cart_number_checksum_validation(self, value):
            raise forms.ValidationError(self.error_messages['invalid'])
        if (value and not self.valid_types is None
                and not card_type in self.valid_types):
            valid_types = map(issuer_name, self.valid_types)
            error_message = self.error_messages['invalid_type'] % {
                'valid_types': ', '.join(valid_types)
            }
            raise forms.ValidationError(error_message)

    @staticmethod
    def cart_number_checksum_validation(cls, number):
        digits = []
        even = False
        if not number.isdigit():
            return False
        for digit in reversed(number):
            digit = ord(digit) - ord('0')
            if even:
                digit *= 2
                if digit >= 10:
                    digit = digit % 10 + digit // 10
            digits.append(digit)
            even = not even
        return sum(digits) % 10 == 0 if digits else False


class CreditCardExpiryField(forms.MultiValueField):

    EXP_MONTH = [(str(x), '%02d' % (x,)) for x in range(1, 13)]
    EXP_YEAR = [(str(x), str(x)) for x in range(date.today().year,
                                                date.today().year + 15)]

    default_error_messages = {
        'invalid_month': 'Enter a valid month.',
        'invalid_year': 'Enter a valid year.'}

    def __init__(self, *args, **kwargs):
        errors = self.default_error_messages.copy()
        if 'error_messages' in kwargs:
            errors.update(kwargs['error_messages'])

        fields = (
            forms.ChoiceField(
                choices=[('', _('Month'))] + self.EXP_MONTH,
                error_messages={'invalid': errors['invalid_month']},
                widget=forms.Select(
                    attrs={'autocomplete': 'cc-exp-month'})),
            forms.ChoiceField(
                choices=[('', _('Year'))] + self.EXP_YEAR,
                error_messages={'invalid': errors['invalid_year']},
                widget=forms.Select(
                    attrs={'autocomplete': 'cc-exp-year'})),
        )

        super(CreditCardExpiryField, self).__init__(fields, *args, **kwargs)
        self.widget = CreditCardExpiryWidget(widgets=[fields[0].widget,
                                                      fields[1].widget])

    def clean(self, value):
        exp = super(CreditCardExpiryField, self).clean(value)
        if date.today() > exp:
            raise forms.ValidationError(
                "The expiration date you entered is in the past.")
        return exp

    def compress(self, data_list):
        if data_list:
            if data_list[1] in forms.fields.EMPTY_VALUES:
                error = self.error_messages['invalid_year']
                raise forms.ValidationError(error)
            if data_list[0] in forms.fields.EMPTY_VALUES:
                error = self.error_messages['invalid_month']
                raise forms.ValidationError(error)
            year = int(data_list[1])
            month = int(data_list[0])
            # find last day of the month
            day = monthrange(year, month)[1]
            return date(year, month, day)
        return None


class CreditCardVerificationField(forms.CharField):

    widget = forms.TextInput(attrs={'autocomplete': 'cc-csc'})
    default_error_messages = {
        'invalid': _('Enter a valid security number.')}

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = kwargs.pop('max_length', 4)
        super(CreditCardVerificationField, self).__init__(*args, **kwargs)

    def validate(self, value):
        if value in validators.EMPTY_VALUES and self.required:
            raise forms.ValidationError(self.error_messages['required'])
        if value and not re.match('^[0-9]{3,4}$', value):
            raise forms.ValidationError(self.error_messages['invalid'])


class CreditCardNameField(forms.CharField):

    widget = forms.TextInput(attrs={'autocomplete': 'cc-name'})



from __future__ import unicode_literals

import logging
import requests
from datetime import timedelta
from functools import wraps
import json
from decimal import Decimal, ROUND_HALF_UP
from requests.exceptions import HTTPError

try:
    from itertools import ifilter as filter
except ImportError:
    pass

from django.http import HttpResponseForbidden
from django.shortcuts import redirect
from django.utils import timezone

from .forms import PaymentForm
from .. import (
    BasicProvider, get_credit_card_issuer, PaymentError, RedirectNeeded)

# Get an instance of a logger
logger = logging.getLogger(__name__)

CENTS = Decimal('0.01')


class UnauthorizedRequest(Exception):
    pass


def authorize(fun):
    @wraps(fun)
    def wrapper(*args, **kwargs):
        self = args[0]
        self.access_token = self.get_access_token()
        try:
            response = fun(*args, **kwargs)
        except HTTPError as e:
            if e.response.status_code == 401:
                last_auth_response = self.get_last_response(is_auth=True)
                if 'access_token' in last_auth_response:
                    del last_auth_response['access_token']
                    self.set_response_data(last_auth_response, is_auth=True)
                self.access_token = self.get_access_token()
                response = fun(*args, **kwargs)
            raise
        return response

    return wrapper


class PaypalProvider(BasicProvider):
    '''
    paypal.com payment provider
    '''
    def set_response_data(self, response, is_auth=False):
        extra_data = json.loads(self.payment.extra_data or '{}')
        if is_auth:
            extra_data['auth_response'] = response
        else:
            extra_data['response'] = response
            if 'links' in response:
                extra_data['links'] = dict(
                    (link['rel'], link) for link in response['links'])
        self.payment.extra_data = json.dumps(extra_data)

    def set_response_links(self, links):
        extra_data = json.loads(self.payment.extra_data or '{}')
        extra_data['links'] = dict((link['rel'], link) for link in links)
        self.payment.extra_data = json.dumps(extra_data)

    def set_error_data(self, error):
        extra_data = json.loads(self.payment.extra_data or '{}')
        extra_data['error'] = error
        self.payment.extra_data = json.dumps(extra_data)

    @property
    def links(self):
        extra_data = json.loads(self.payment.extra_data or '{}')
        links = extra_data.get('links', {})
        return links

    @authorize
    def post(self, *args, **kwargs):
        kwargs['headers'] = {
            'Content-Type': 'application/json',
            'Authorization': self.access_token}
        if 'data' in kwargs:
            kwargs['data'] = json.dumps(kwargs['data'])
        response = requests.post(*args, **kwargs)
        try:
            data = response.json()
        except ValueError:
            data = {}
        if 400 <= response.status_code <= 500:
            self.set_error_data(data)
            logger.debug(data)
            message = 'Paypal error'
            if response.status_code == 400:
                error_data = response.json()
                logger.warning(message, extra={
                    'response': error_data,
                    'status_code': response.status_code})
                message = error_data.get('message', message)
            else:
                logger.warning(
                    message, extra={'status_code': response.status_code})
            self.payment.change_status('error', message)
            raise PaymentError(message)
        else:
            self.set_response_data(data)
        return data

    def get_last_response(self, is_auth=False):
        extra_data = json.loads(self.payment.extra_data or '{}')
        if is_auth:
            return extra_data.get('auth_response', {})
        return extra_data.get('response', {})

    def __init__(self, *args, **kwargs):
        self.secret = kwargs.pop('secret')
        self.client_id = kwargs.pop('client_id')
        self.endpoint = kwargs.pop(
            'endpoint', 'https://api.sandbox.paypal.com')
        self.oauth2_url = self.endpoint + '/v1/oauth2/token'
        self.payments_url = self.endpoint + '/v1/payments/payment'
        self.payment_execute_url = self.payments_url + '/%(id)s/execute/'
        self.payment_refund_url = (
            self.endpoint + '/v1/payments/capture/{captureId}/refund')
        super(PaypalProvider, self).__init__(*args, **kwargs)

    def get_access_token(self):
        last_auth_response = self.get_last_response(is_auth=True)
        created = self.payment.created
        now = timezone.now()
        if ('access_token' in last_auth_response and
                'expires_in' in last_auth_response and
                (created + timedelta(
                    seconds=last_auth_response['expires_in'])) > now):
            return '%s %s' % (last_auth_response['token_type'],
                              last_auth_response['access_token'])
        else:
            headers = {'Accept': 'application/json',
                       'Accept-Language': 'en_US'}
            post = {'grant_type': 'client_credentials'}
            response = requests.post(self.oauth2_url, data=post,
                                     headers=headers,
                                     auth=(self.client_id, self.secret))
            response.raise_for_status()
            data = response.json()
            last_auth_response.update(data)
            self.set_response_data(last_auth_response, is_auth=True)
            return '%s %s' % (data['token_type'], data['access_token'])

    def get_link(self, name, data):
        try:
            links = filter(lambda url: url['rel'] == name, data['links'])
        except KeyError:
            return None
        return next(links)['href']

    def get_transactions_items(self):
        for purchased_item in self.payment.get_purchased_items():
            price = purchased_item.price.quantize(
                CENTS, rounding=ROUND_HALF_UP)
            item = {'name': purchased_item.name[:127],
                    'quantity': str(purchased_item.quantity),
                    'price': str(price),
                    'currency': purchased_item.currency,
                    'sku': purchased_item.sku}
            yield item

    def get_transactions_data(self):
        items = list(self.get_transactions_items())
        sub_total = (
            self.payment.total - self.payment.delivery - self.payment.tax)
        sub_total = sub_total.quantize(CENTS, rounding=ROUND_HALF_UP)
        total = self.payment.total.quantize(CENTS, rounding=ROUND_HALF_UP)
        tax = self.payment.tax.quantize(CENTS, rounding=ROUND_HALF_UP)
        delivery = self.payment.delivery.quantize(
            CENTS, rounding=ROUND_HALF_UP)
        data = {
            'intent': 'sale' if self._capture else 'authorize',
            'transactions': [{'amount': {
                'total': str(total),
                'currency': self.payment.currency,
                'details': {
                    'subtotal': str(sub_total),
                    'tax': str(tax),
                    'shipping': str(delivery)}},
                'item_list': {'items': items},
                'description': self.payment.description}]}
        return data

    def get_product_data(self, extra_data=None):
        return_url = self.get_return_url()
        data = self.get_transactions_data()
        data['redirect_urls'] = {'return_url': return_url,
                                 'cancel_url': return_url}
        data['payer'] = {'payment_method': 'paypal'}
        return data

    def get_form(self, data=None):
        if not self.payment.id:
            self.payment.save()
        redirect_to = self.links.get('approval_url')
        if not redirect_to:
            payment = self.create_payment()
            self.payment.transaction_id = payment['id']
            redirect_to = self.links['approval_url']
        self.payment.change_status('waiting')
        raise RedirectNeeded(redirect_to['href'])

    def process_data(self, request):
        success_url = self.payment.get_success_url()
        if not 'token' in request.GET:
            return HttpResponseForbidden('FAILED')
        payer_id = request.GET.get('PayerID')
        if not payer_id:
            if self.payment.status != 'confirmed':
                self.payment.change_status('rejected')
                return redirect(self.payment.get_failure_url())
            else:
                return redirect(success_url)
        payment = self.execute_payment(payer_id)
        related_resources = payment['transactions'][0]['related_resources'][0]
        resource_key = 'sale' if self._capture else 'authorization'
        authorization_links = related_resources[resource_key]['links']
        self.set_response_links(authorization_links)
        self.payment.attrs.payer_info = payment['payer']['payer_info']
        if self._capture:
            self.payment.captured_amount = self.payment.total
            self.payment.change_status('confirmed')
        else:
            self.payment.change_status('preauth')
        return redirect(success_url)

    def create_payment(self, extra_data=None):
        product_data = self.get_product_data(extra_data)
        payment = self.post(self.payments_url, data=product_data)
        return payment

    def execute_payment(self, payer_id):
        post = {'payer_id': payer_id}
        execute_url = self.links['execute']['href']
        return self.post(execute_url, data=post)

    def get_amount_data(self, amount=None):
        return {
            'currency': self.payment.currency,
            'total': str(amount.quantize(
                CENTS, rounding=ROUND_HALF_UP))}

    def capture(self, amount=None):
        if amount is None:
            amount = self.payment.total
        amount_data = self.get_amount_data(amount)
        capture_data = {
            'amount': amount_data,
            'is_final_capture': True
        }
        url = self.links['capture']['href']
        try:
            capture = self.post(url, data=capture_data)
        except HTTPError as e:
            try:
                error = e.response.json()
            except ValueError:
                error = {}
            if error.get('name') != 'AUTHORIZATION_ALREADY_COMPLETED':
                raise e
            capture = {'state': 'completed'}
        state = capture['state']
        if state in [
                'completed', 'partially_captured', 'partially_refunded']:
            return amount
        elif state == 'pending':
            self.payment.change_status('waiting')
        elif state == 'refunded':
            self.payment.change_status('refunded')
            raise PaymentError('Payment already refunded')

    def release(self):
        url = self.links['void']['href']
        self.post(url)

    def refund(self, amount=None):
        if amount is None:
            amount = self.payment.captured_amount
        amount_data = self.get_amount_data(amount)
        refund_data = {'amount': amount_data}
        url = self.links['refund']['href']
        self.post(url, data=refund_data)
        self.payment.change_status('refunded')
        return amount


class PaypalCardProvider(PaypalProvider):
    '''
    paypal.com credit card payment provider
    '''

    def get_form(self, data=None):
        if self.payment.status == 'waiting':
            self.payment.change_status('input')
        form = PaymentForm(data, provider=self, payment=self.payment)
        if form.is_valid():
            raise RedirectNeeded(self.payment.get_success_url())
        return form

    def get_product_data(self, extra_data):
        data = self.get_transactions_data()
        year = extra_data['expiration'].year
        month = extra_data['expiration'].month
        number = extra_data['number']
        card_type, _card_issuer = get_credit_card_issuer(number)
        credit_card = {'number': number,
                       'type': card_type,
                       'expire_month': month,
                       'expire_year': year}
        if 'cvv2' in extra_data and extra_data['cvv2']:
            credit_card['cvv2'] = extra_data['cvv2']
        data['payer'] = {'payment_method': 'credit_card',
                         'funding_instruments': [{'credit_card': credit_card}]}
        return data

    def process_data(self, request):
        return HttpResponseForbidden('FAILED')



from __future__ import unicode_literals

from django import forms
import stripe

from ..forms import PaymentForm as BasePaymentForm
from .widgets import StripeWidget
from . import RedirectNeeded


class PaymentForm(BasePaymentForm):

    charge = None

    def __init__(self, *args, **kwargs):
        super(PaymentForm, self).__init__(*args, **kwargs)
        widget = StripeWidget(provider=self.provider, payment=self.payment)
        self.fields['stripeToken'] = forms.CharField(widget=widget)
        if self.is_bound and not self.data.get('stripeToken'):
            self.payment.change_status('rejected')
            raise RedirectNeeded(self.payment.get_failure_url())

    def clean(self):
        data = self.cleaned_data

        if not self.errors and not self.payment.transaction_id:
            stripe.api_key = self.provider.secret_key
            try:
                self.charge = stripe.Charge.create(
                    capture=False,
                    amount= int(self.payment.total * 100),
                    currency=self.payment.currency,
                    card=data['stripeToken'],
                    description='%s %s' % (self.payment.billing_last_name,
                                           self.payment.billing_first_name)
                )
            except stripe.CardError as e:
                # The card has been declined
                self._errors['__all__'] = self.error_class([e.message])
                self.payment.change_status('error')

        return data

    def save(self):
        self.charge.capture()
        self.payment.transaction_id = self.charge.id
        self.payment.change_status('confirmed')



from __future__ import unicode_literals
from decimal import Decimal
import json
from unittest import TestCase

from . import PaypalProvider
from .. import PurchasedItem


CLIENT_ID = 'abc123'
PAYMENT_TOKEN = '5a4dae68-2715-4b1e-8bb2-2c2dbe9255f6'
SECRET = '123abc'
VARIANT = 'wallet'


class Payment(object):

    id = 1
    description = 'payment'
    currency = 'USD'
    delivery = Decimal(10)
    status = 'waiting'
    tax = Decimal(10)
    token = PAYMENT_TOKEN
    total = Decimal(100)
    variant = VARIANT

    def change_status(self, status):
        self.status = status

    def get_failure_url(self):
        return 'http://cancel.com'

    def get_process_url(self):
        return 'http://example.com'

    def get_purchased_items(self):
        return [
            PurchasedItem(
                name='foo', quantity=Decimal('10'), price=Decimal('20'),
                currency='USD', sku='bar')]

    def get_success_url(self):
        return 'http://success.com'


class TestPaypalProvider(TestCase):

    def test_payload_serializable(self):
        payment = Payment()
        provider = PaypalProvider(payment, secret=SECRET, client_id=CLIENT_ID)
        json.dumps(provider.get_product_data())



from __future__ import unicode_literals

from django.forms.widgets import HiddenInput


class WalletWidget(HiddenInput):

    def __init__(self, provider, *args, **kwargs):

        kwargs['attrs'] = {
            'id': 'google-wallet-id',
            'data-jwt': provider.get_jwt_data(),
            'data-success-url': provider.payment.get_success_url(),
            'data-failure-url': provider.payment.get_failure_url(),
        }
        super(WalletWidget, self).__init__(*args, **kwargs)
        self.js = [provider.library, 'js/payments/wallet.js']

    @property
    def media(self):
        media = super(WalletWidget, self).media
        media._js = self.js
        return media



from __future__ import unicode_literals

from uuid import uuid4

from django import forms
from django.template.loader import render_to_string
from django.utils.translation import ugettext as _

from .. import PaymentError
from ..forms import CreditCardPaymentFormWithName


class FingerprintWidget(forms.HiddenInput):

    def render(self, name, value, attrs=None):
        final_attrs = self.build_attrs(attrs, type=self.input_type, name=name)
        final_attrs['session_id'] = value
        return render_to_string(
            'payments/cybersource_fingerprint.html', final_attrs)


class FingerprintInput(forms.CharField):

    widget = FingerprintWidget
    hidden_widget = FingerprintWidget

    def __init__(self, org_id, merchant_id, fingerprint_url,  *args, **kwargs):
        self.org_id = org_id
        self.merchant_id = merchant_id
        self.fingerprint_url = fingerprint_url
        super(FingerprintInput, self).__init__(*args, **kwargs)

    def widget_attrs(self, widget):
        attrs = super(FingerprintInput, self).widget_attrs(widget=widget)
        attrs['org_id'] = self.org_id
        attrs['merchant_id'] = self.merchant_id
        attrs['fingerprint_url'] = self.fingerprint_url
        return attrs


class PaymentForm(CreditCardPaymentFormWithName):

    def __init__(self, *args, **kwargs):
        super(PaymentForm, self).__init__(*args, **kwargs)
        if self.provider.org_id:
            try:
                fingerprint_id = self.payment.attrs.fingerprint_session_id
            except KeyError:
                fingerprint_id = str(uuid4())
            self.fields['fingerprint'] = FingerprintInput(
                label=_('fingerprint'), org_id=self.provider.org_id,
                initial=fingerprint_id,
                merchant_id=self.provider.merchant_id,
                fingerprint_url=self.provider.fingerprint_url)

    def clean(self):
        cleaned_data = super(PaymentForm, self).clean()
        if not self.errors:
            if self.provider.org_id:
                fingerprint = cleaned_data['fingerprint']
                self.payment.attrs.fingerprint_session_id = fingerprint
            if not self.payment.transaction_id:
                try:
                    self.provider.charge(cleaned_data)
                except PaymentError as e:
                    self._errors['__all__'] = self.error_class([e.args[0]])
            else:
                msg = _('This payment has already been processed.')
                self._errors['__all__'] = self.error_class([msg])
        return cleaned_data



from __future__ import unicode_literals

from django.core.exceptions import ImproperlyConfigured
from django.shortcuts import redirect

from .. import BasicProvider, RedirectNeeded
from .forms import PaymentForm


class StripeProvider(BasicProvider):

    def __init__(self, *args, **kwargs):
        self.secret_key = kwargs.pop('secret_key')
        self.public_key = kwargs.pop('public_key')
        self.image = kwargs.pop('image', '')
        self.name = kwargs.pop('name', '')
        super(StripeProvider, self).__init__(*args, **kwargs)
        if not self._capture:
            raise ImproperlyConfigured(
                'Stripe does not support pre-authorization.')

    def get_form(self, data=None):
        kwargs = {
            'data': data,
            'payment': self.payment,
            'provider': self,
            'action': '',
            'hidden_inputs': False}
        form = PaymentForm(**kwargs)

        if form.is_valid():
            form.save()
            raise RedirectNeeded(self.payment.get_success_url())
        else:
            self.payment.change_status('input')
        return form

    def process_data(self, request):
        if self.payment.status == 'confirmed':
            return redirect(self.payment.get_success_url())
        return redirect(self.payment.get_failure_url())



#!/usr/bin/env python
from setuptools import setup
import os
import sys

os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'test_settings')

PACKAGES = [
    'payments',
    'payments.authorizenet',
    'payments.cybersource',
    'payments.dummy',
    'payments.dotpay',
    'payments.paypal',
    'payments.sagepay',
    'payments.stripe',
    'payments.wallet']

REQUIREMENTS = [
    'Django>=1.5',
    'pycrypto>=2.6',
    'PyJWT',
    'requests>=1.2.0',
    'stripe>=1.9.8',
    'suds-jurko>=0.6']

if sys.version_info < (3,):
    PACKAGES += ['payments.braintree']
    REQUIREMENTS += ['braintree>=2.24.1']

setup(
    name='django-payments',
    author='Mirumee Software',
    author_email='hello@mirumee.com',
    description='Universal payment handling for Django',
    version='0.6.6',
    url='http://github.com/mirumee/django-payments',
    packages=PACKAGES,
    include_package_data=True,
    classifiers=[
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Framework :: Django',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules'],
    install_requires=REQUIREMENTS,
    tests_require=['mock'],
    test_suite='payments.tests',
    zip_safe=False)



'''
This module is responsible for automatic processing of provider callback
data (asynchronous transaction updates).
'''
from __future__ import unicode_literals

from django.conf.urls import patterns, url
from django.http import Http404
from django.shortcuts import get_object_or_404
from django.views.decorators.csrf import csrf_exempt
try:
    from django.db.transaction import atomic
except ImportError:
    def atomic(func):
        return func

from . import factory, get_payment_model, provider_factory


@csrf_exempt
@atomic
def process_data(request, token, provider=None):
    '''
    Calls process_data of an appropriate provider.

    Raises Http404 if variant does not exist.
    '''
    Payment = get_payment_model()
    payment = get_object_or_404(Payment, token=token)
    if provider:
        provider.payment = payment
    else:
        try:
            provider = factory(payment)
        except ValueError:
            raise Http404('No such payment')
    return provider.process_data(request)


@csrf_exempt
@atomic
def static_callback(request, variant):

    try:
        provider = provider_factory(variant)
    except ValueError:
        raise Http404('No such provider')

    token = provider.get_token_from_request(request)
    if not token:
        raise Http404('Invalid response')
    return process_data(request, token, provider)


urlpatterns = patterns(
    '',
    url(r'^process/(?P<token>[0-9a-z]{8}-[0-9a-z]{4}-'
        '[0-9a-z]{4}-[0-9a-z]{4}-[0-9a-z]{12})/$', process_data,
        name='process_payment'),
    url(r'^process/(?P<variant>[a-z-]+)/$', static_callback,
        name='static_process_payment'),)



from __future__ import unicode_literals

import braintree

from ..forms import CreditCardPaymentFormWithName


class BraintreePaymentForm(CreditCardPaymentFormWithName):

    transaction_id = None

    def clean(self):
        data = self.cleaned_data

        if not self.errors and not self.payment.transaction_id:
            result = braintree.Transaction.sale({
                'amount': str(self.payment.total),
                'billing': self.get_billing_data(),
                'credit_card': self.get_credit_card_clean_data(),
                'customer': self.get_customer_data(),
                'options': {
                    'submit_for_settlement': True
                },
                'order_id': self.payment.description
            })

            if result.is_success:
                self.transaction_id = result.transaction.id
            else:
                self._errors['__all__'] = self.error_class([result.message])
                self.payment.change_status('error')

        return data

    def get_credit_card_clean_data(self):
        if self.cleaned_data:
            return {
                'number': self.cleaned_data.get('number'),
                'cvv': self.cleaned_data.get('cvv2'),
                'cardholder_name': self.cleaned_data.get('name'),
                'expiration_month': self.cleaned_data.get('expiration').month,
                'expiration_year': self.cleaned_data.get('expiration').year}

    def get_billing_data(self):
        return {
            'first_name': self.payment.billing_first_name,
            'last_name': self.payment.billing_last_name,
            'street_address': self.payment.billing_address_1,
            'extended_address': self.payment.billing_address_2,
            'locality': self.payment.billing_city,
            'region': self.payment.billing_country_area,
            'postal_code': self.payment.billing_postcode,
            'country_code_alpha2': self.payment.billing_country_code}

    def get_customer_data(self):
        return {
            'first_name': self.payment.billing_first_name,
            'last_name': self.payment.billing_last_name}

    def save(self):
        braintree.Transaction.submit_for_settlement(self.transaction_id)
        self.payment.transaction_id = self.transaction_id
        self.payment.change_status('confirmed')



from unittest import TestCase

import django

if hasattr(django, 'setup'):
    django.setup()

from . import provider_factory
from .dotpay.tests import TestDotpayProvider
from .paypal.tests import TestPaypalProvider
from .stripe.tests import TestStripeProvider
from .wallet.tests import TestGoogleWalletProvider
from .dummy.tests import TestDummy3DSProvider


__all__ = ['TestDotpayProvider', 'TestGoogleWalletProvider',
           'TestPaypalProvider', 'TestStripeProvider', 'TestDummy3DSProvider']


class TestProviderFactory(TestCase):

    def test_provider_factory(self):
        provider_factory('default')



from __future__ import unicode_literals

try:
    from collections import OrderedDict
except ImportError:
    from django.utils.datastructures import SortedDict as OrderedDict

from django import forms
from django.utils.translation import ugettext_lazy as _

from .fields import (CreditCardNumberField, CreditCardExpiryField,
                     CreditCardVerificationField, CreditCardNameField)


class PaymentForm(forms.Form):
    '''
    Payment form, suitable for Django templates.

    When displaying the form remeber to use *action* and *method*.
    '''

    #: Form action URL for template use
    action = ''
    #: Form method for template use, either "get" or "post"
    method = 'post'

    def __init__(self, data=None, action=None, method='post', provider=None,
                 payment=None, hidden_inputs=True, autosubmit=False):
        if hidden_inputs and data is not None:
            super(PaymentForm, self).__init__(auto_id=False)
            for key, val in data.items():
                widget = forms.widgets.HiddenInput()
                self.fields[key] = forms.CharField(initial=val, widget=widget)
        else:
            super(PaymentForm, self).__init__(data=data)
        self.action = action
        self.autosubmit = autosubmit
        self.method = method
        self.provider = provider
        self.payment = payment


class CreditCardPaymentForm(PaymentForm):

    number = CreditCardNumberField(label=_('Card Number'), max_length=32,
                                   required=True)
    expiration = CreditCardExpiryField()
    cvv2 = CreditCardVerificationField(
        label=_('CVV2 Security Number'), required=False, help_text=_(
            'Last three digits located on the back of your card.'
            ' For American Express the four digits found on the front side.'))

    def __init__(self, *args, **kwargs):
        super(CreditCardPaymentForm, self).__init__(
            hidden_inputs=False, *args,  **kwargs)
        if hasattr(self, 'VALID_TYPES'):
            self.fields['number'].valid_types = self.VALID_TYPES


class CreditCardPaymentFormWithName(CreditCardPaymentForm):

    name = CreditCardNameField(label=_('Name on Credit Card'), max_length=128)

    def __init__(self, *args, **kwargs):
        super(CreditCardPaymentFormWithName, self).__init__(*args, **kwargs)
        name_field = self.fields.pop('name')
        fields = OrderedDict({'name': name_field})
        fields.update(self.fields)
        self.fields = fields
